"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
const utils_1 = require("@nodeswork/utils");
const model = require("./model");
exports.READONLY = 'READONLY';
exports.AUTOGEN = 'AUTOGEN';
class KoaMiddlewares extends model.Model {
    static createMiddleware(options) {
        let self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        async function create(ctx, next) {
            let model = self;
            let doc = _.extend({}, ctx.request.body, ctx.overrides && ctx.overrides.doc);
            let omits = _.union(options.omits, self.schema.api.AUTOGEN);
            doc = _.omit(doc, omits);
            let discriminatorKey = self.schema.options.discriminatorKey;
            if (discriminatorKey) {
                let modelName = doc[discriminatorKey];
                if (modelName) {
                    model = self.db.model(modelName);
                }
                if (!options.allowCreateFromParentModel && !modelName) {
                    throw new utils_1.NodesworkError('required field is missing', {
                        responseCode: 422,
                        path: discriminatorKey,
                    });
                }
                if (!model) {
                    throw new utils_1.NodesworkError('invalid value', {
                        responseCode: 422,
                        path: discriminatorKey,
                    });
                }
            }
            ctx[options.target] = doc;
            if (options.triggerNext) {
                await next();
            }
            doc = ctx[options.target];
            let object = (await model.create(doc));
            if (options.project || options.level) {
                object = await self.findById(object._id, options.project, {
                    level: options.level,
                });
            }
            ctx[options.target] = object;
            if (options.populate) {
                await model.populate(object, options.populate);
            }
            if (!options.noBody) {
                ctx.body = options.transform(object);
            }
        }
        Object.defineProperty(create, 'name', {
            value: `${self.modelName}#createMiddleware`,
            writable: false,
        });
        return create;
    }
    static getMiddleware(options) {
        let self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        async function get(ctx, next) {
            let query = ctx.overrides && ctx.overrides.query || {};
            query._id = ctx.params[options.field];
            let queryOption = {};
            if (options.level) {
                queryOption.level = options.level;
            }
            let queryPromise = self.findOne(query, options.project, queryOption);
            if (options.populate) {
                queryPromise = queryPromise.populate(options.populate);
            }
            let object = await queryPromise;
            ctx[options.target] = object;
            if (options.triggerNext) {
                await next();
            }
            if (!options.nullable && object == null) {
                throw new utils_1.NodesworkError('not found', {
                    responseCode: 404
                });
            }
            if (!options.noBody) {
                ctx.body = options.transform(ctx[options.target]);
            }
        }
        Object.defineProperty(get, 'name', {
            value: `${self.modelName}#getMiddleware`,
            writable: false,
        });
        return get;
    }
    static findMiddleware(options) {
        let self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        if (options.pagination) {
            _.defaults(options.pagination, DEFAULT_FIND_PAGINATION_OPTIONS);
        }
        const defaultPagination = {
            page: 0,
            size: options.pagination ? options.pagination.size : 0,
        };
        async function find(ctx, next) {
            let query = ctx.overrides && ctx.overrides.query || {};
            let queryOption = {};
            let pagination = null;
            if (options.pagination) {
                // pagination          = VALIDATE_QUERY_PAGINATION(ctx.request.query);
                pagination = ctx.request.query;
                _.defaults(pagination, defaultPagination);
                queryOption.skip = pagination.page * pagination.size;
                queryOption.limit = pagination.size;
            }
            if (options.level) {
                queryOption.level = options.level;
            }
            let queryPromise = self.find(query, options.project, queryOption);
            if (options.populate) {
                queryPromise = queryPromise.populate(options.populate);
            }
            let object = await queryPromise;
            ctx[options.target] = object;
            if (options.triggerNext) {
                await next();
            }
            if (pagination) {
                let totalPage = Math.floor((await self.find(query).count() - 1) / pagination.size + 1);
                ctx.response.set('total_page', totalPage.toString());
            }
            if (!options.noBody) {
                let body = ctx[options.target];
                for (let i in body) {
                    body[i] = options.transform(body[i]);
                }
                ctx.body = body;
            }
        }
        Object.defineProperty(find, 'name', {
            value: `${self.modelName}#findMiddleware`,
            writable: false,
        });
        return find;
    }
    static updateMiddleware(options) {
        let self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        async function update(ctx, next) {
            let query = ctx.overrides && ctx.overrides.query || {};
            query._id = ctx.params[options.field];
            let queryOption = {
                new: true,
                fields: options.project,
                level: options.level,
            };
            let omits = _.union(['_id'], options.omits, self.schema.api.READONLY, self.schema.api.AUTOGEN);
            let update = {
                $set: _.omit(ctx.request.body, omits),
            };
            var object = await self.findOneAndUpdate(query, update, queryOption);
            ctx[options.target] = object;
            if (options.triggerNext) {
                await next();
            }
            if (!options.noBody) {
                let body = ctx[options.target];
                for (let i in body) {
                    body[i] = options.transform(body[i]);
                }
                ctx.body = body;
            }
        }
        Object.defineProperty(update, 'name', {
            value: `${self.modelName}#updateMiddleware`,
            writable: false,
        });
        return update;
    }
    static deleteMiddleware(options) {
        let self = this.cast();
        _.defaults(options, DEFAULT_COMMON_OPTIONS);
        async function del(ctx, next) {
            let query = ctx.overrides && ctx.overrides.query || {};
            query._id = ctx.params[options.field];
            let queryOption = {};
            let queryPromise = self.findOne(query, undefined, queryOption);
            let object = await queryPromise;
            ctx[options.target] = object;
            if (options.triggerNext) {
                await next();
            }
            object = ctx[options.target];
            if (!options.nullable && object == null) {
                throw new utils_1.NodesworkError('not found', {
                    responseCode: 404
                });
            }
            if (object) {
                await object.delete();
            }
            ctx.status = 204;
        }
        Object.defineProperty(del, 'name', {
            value: `${self.modelName}#deleteMiddleware`,
            writable: false,
        });
        return del;
    }
}
exports.KoaMiddlewares = KoaMiddlewares;
const DEFAULT_COMMON_OPTIONS = {
    target: 'object',
    transform: _.identity,
};
const DEFAULT_FIND_PAGINATION_OPTIONS = {
    size: 20,
    sizeChoices: [20, 50, 100, 200],
};
const VALIDATE_QUERY_PAGINATION = utils_1.validator2.compile({
    page: [],
    size: [],
});
KoaMiddlewares.Plugin({
    fn: apiLevel,
});
function apiLevel(schema, options) {
    if (schema.api == null) {
        schema.api = {
            READONLY: [],
            AUTOGEN: []
        };
    }
    schema.eachPath(function (pathname, schemaType) {
        if ([exports.READONLY, exports.AUTOGEN].indexOf(schemaType.options.api) < 0) {
            return;
        }
        for (let s = schema; s != null; s = s.parentSchema) {
            s.api[schemaType.options.api] = _.union(s.api[schemaType.options.api], [pathname]);
        }
    });
}

//# sourceMappingURL=koa.js.map
